@@ -19,7 +19,6 @@ import requests
 import textwrap
 
 import datetime as dt
-import cloudvolume as cv
 import numpy as np
 import pandas as pd
 
@@ -33,12 +32,6 @@ from .. import xform
 
 from .utils import parse_volume, FLYWIRE_DATASETS, get_chunkedgraph_secret
 
-try:
-    import skeletor as sk
-except ImportError:
-    sk = None
-except BaseException:
-    raise
 
 __all__ = ['fetch_edit_history', 'fetch_leaderboard', 'locs_to_segments',
            'locs_to_supervoxels', 'skid_to_id', 'update_ids',
@@ -171,7 +164,7 @@ def fetch_edit_history(x, dataset='production', progress=True, max_threads=4):
     futures = []
     for id in x:
         dataset = FLYWIRE_DATASETS.get(dataset, dataset)
-        url = f'https://prodv1.flywire-daf.com/segmentation/api/v1/table/{dataset}/root/{id}/tabular_change_log'
+        url = f'https://prod.flywire-daf.com/segmentation/api/v1/table/{dataset}/root/{id}/tabular_change_log'
         f = future_session.get(url, params=None)
         futures.append(f)
 
@@ -309,7 +302,6 @@ def supervoxels_to_roots(x, use_cache=False, dataset='production'):
     >>> from fafbseg import flywire
     >>> flywire.supervoxels_to_roots(78321855915861142)
 
-
     """
     # Make sure we are working with an array of integers
     x = navis.utils.make_iterable(x).astype(np.int64, copy=False)
@@ -513,6 +505,7 @@ def locs_to_segments(locs, root_ids=True, dataset='production',
 
 
 def skid_to_id(x,
+               sample=None,
                dataset='production',
                progress=True, **kwargs):
     """Find the flywire ID(s) corresponding to given CATMAID skeleton ID(s).
@@ -528,6 +521,10 @@ def skid_to_id(x,
     x :             int | list-like | str | TreeNeuron/List
                     Anything that's not a TreeNeuron/List will be passed
                     directly to ``pymaid.get_neuron``.
+    sample :        int | float, optional
+                    Number (>= 1) or fraction (< 1) of super nodes to sample
+                    to find flywire IDs. If ``None`` (default), will use all
+                    nodes.
     dataset :       str | CloudVolume
                     Against which flywire dataset to query::
                         - "production" (current production dataset, fly_v31)
@@ -540,30 +537,40 @@ def skid_to_id(x,
     pandas.DataFrame
                     Mapping of flywire IDs to skeleton IDs with confidence::
 
-                      flywire_id   skeleton_id   confidence
+                      skeleton_id   flywire_id   confidence
                     0
                     1
 
     """
-    vol = parse_volume(dataset, **kwargs)
-
     if not isinstance(x, (navis.TreeNeuron, navis.NeuronList)):
         x = pymaid.get_neuron(x)
 
-    if isinstance(x, navis.TreeNeuron):
-        nodes = x.nodes[['x', 'y', 'z']].copy()
-        nodes['skeleton_id'] = x.id
-    elif isinstance(x, navis.NeuronList):
-        nodes = x.nodes[['x', 'y', 'z']].copy()
+    if isinstance(x, navis.NeuronList) and len(x) == 1:
+        x = x[0]
+
+    if isinstance(x, navis.NeuronList):
+        res = []
+        for n in navis.config.tqdm(x, desc='Searching',
+                                   disable=not progress,
+                                   leave=False):
+            res.append(skid_to_id(n, dataset=dataset))
+        return pd.concat(res, axis=0)
+    elif isinstance(x, navis.TreeNeuron):
+        nodes = x.nodes[['x', 'y', 'z']]
+        if sample:
+            if sample < 1:
+                nodes = nodes.sample(frac=sample, random_state=1985)
+            else:
+                nodes = nodes.sample(n=sample, random_state=1985)
     else:
-        raise TypeError(f'Unable to data of type "{type(x)}"')
+        raise TypeError(f'Unable to use data of type "{type(x)}"')
 
     # XForm coordinates from FAFB14 to FAFB14.1
     xformed = xform.fafb14_to_flywire(nodes[['x', 'y', 'z']].values,
                                       coordinates='nm')
 
     # Get the root IDs for each of these locations
-    roots = locs_to_segments(xformed, coordinates='nm')
+    roots = locs_to_segments(xformed, coordinates='nm', dataset=dataset)
 
     # Drop zeros
     roots = roots[roots != 0]
@@ -574,7 +581,7 @@ def skid_to_id(x,
     # Get sorted indices
     sort_ix = np.argsort(counts)
 
-    # New Id is the most frequent ID
+    # The "correct" ID is assumed to be the most frequent ID
     new_id = unique[sort_ix[-1]]
 
     # Confidence is the difference between the top and the 2nd most frequent ID
@@ -584,8 +591,8 @@ def skid_to_id(x,
     else:
         conf = 1
 
-    return pd.DataFrame([[x.id, new_id, conf, x.id != new_id]],
-                        columns=['old_id', 'new_id', 'confidence', 'changed'])
+    return pd.DataFrame([[x.id, new_id, conf]],
+                        columns=['skeleton_id', 'flywire_id', 'confidence'])
 
 
 def is_latest_root(id, dataset='production', **kwargs):
@@ -625,7 +632,7 @@ def is_latest_root(id, dataset='production', **kwargs):
     token = get_chunkedgraph_secret()
     session.headers['Authorization'] = f"Bearer {token}"
 
-    url = f'https://prodv1.flywire-daf.com/segmentation/api/v1/table/{dataset}/is_latest_roots?int64_as_str=1'
+    url = f'https://prod.flywire-daf.com/segmentation/api/v1/table/{dataset}/is_latest_roots?int64_as_str=1'
     post = {'node_ids': id.tolist()}
     r = session.post(url, json=post)
 
