@@ -15,21 +15,49 @@
 
 import navis
 
+import datetime as dt
 import numpy as np
 import pandas as pd
 
-from .segmentation import roots_to_supervoxels, supervoxels_to_roots, is_latest_root
-from .utils import parse_root_ids
+from functools import partial
+from tqdm.auto import trange
+
+from .segmentation import is_latest_root
+from .utils import parse_root_ids, get_cave_client
 
 from ..synapses.utils import catmaid_table
 from ..synapses.transmitters import collapse_nt_predictions
-from .. import spine
 
 __all__ = ['fetch_synapses', 'fetch_connectivity', 'predict_transmitter',
            'fetch_adjacency']
 
 
-def predict_transmitter(x, single_pred=False, dataset='production'):
+def synapse_counts(x, batch_size=10, dataset='production'):
+    """Fetch synapse counts for given root IDs.
+
+    Parameters
+    ----------
+    x :             int | list of int | Neuron/List
+                    Either a flywire segment ID (i.e. root ID), a list thereof or
+                    a Neuron/List. For neurons, the ``.id`` is assumed to be the
+                    root ID. If you have a neuron (in FlyWire space) but don't
+                    know its ID, use :func:`fafbseg.flywire.neuron_to_segments`
+                    first.
+    dataset :       str | CloudVolume
+                    Against which flywire dataset to query::
+                        - "production" (current production dataset, fly_v31)
+                        - "sandbox" (i.e. fly_v26)
+
+    Returns
+    -------
+    pandas.DataFrame
+
+    """
+    pass
+
+
+def predict_transmitter(x, single_pred=False, weighted=True, live_query=True,
+                        dataset='production'):
     """Fetch neurotransmitter predictions for neurons.
 
     Based on Eckstein et al. (2020). Uses a service on spine.janelia.org hosted
@@ -50,6 +78,13 @@ def predict_transmitter(x, single_pred=False, dataset='production'):
     single_pred :   bool
                     Whether to only return the highest probability transmitter
                     for each neuron.
+    weighted :      bool
+                    If True, will weight predictions based on confidence: higher
+                    cleft score = more weight.
+    live_query :    bool
+                    Whether to query against the live data or against the latest
+                    materialized table. The latter is useful if you are working
+                    with IDs that you got from another annotation table.
     dataset :       str | CloudVolume
                     Against which flywire dataset to query::
                         - "production" (current production dataset, fly_v31)
@@ -68,18 +103,18 @@ def predict_transmitter(x, single_pred=False, dataset='production'):
     """
     # First get the synapses
     syn = fetch_synapses(x, pre=True, post=False, attach=False, min_score=None,
-                         transmitters=True)
+                         transmitters=True, live_query=live_query)
 
     # Get the predictions
-    return collapse_nt_predictions(syn, single_pred=single_pred, id_col='pre')
+    return collapse_nt_predictions(syn, single_pred=single_pred,
+                                   weighted=weighted, id_col='pre')
 
 
-def fetch_synapses(x, pre=True, post=True, attach=True, min_score=0,
-                   dataset='production', transmitters=False, progress=True):
+def fetch_synapses(x, pre=True, post=True, attach=True, min_score=0, clean=True,
+                   transmitters=False, live_query=True, batch_size=100,
+                   dataset='production', progress=True):
     """Fetch Buhmann et al. (2019) synapses for given neuron(s).
 
-    Uses a service on spine.janelia.org hosted by Eric Perlman and Davi Bock.
-
     Parameters
     ----------
     x :             int | list of int | Neuron/List
@@ -93,12 +128,28 @@ def fetch_synapses(x, pre=True, post=True, attach=True, min_score=0,
     post :          bool
                     Whether to fetch postsynapses for the given neurons.
     transmitters :  bool
-                    Whether to also load neurotransmitter predictions from
-                    Eckstein et al. (2020).
+                    Whether to also load per-synapse neurotransmitter predictions
+                    from Eckstein et al. (2020).
     attach :        bool
                     If True and ``x`` is Neuron/List, the synapses will be added
                     as ``.connectors`` table. For TreeNeurons (skeletons), the
                     synapses will be mapped to the closest node.
+    min_score :     int
+                    Minimum "cleft score". The default of 30 is what Buhmann et al.
+                    used in the paper.
+    clean :         bool
+                    If True, we will perform some clean up of the connectivity
+                    compared with the raw synapse information. Currently, we::
+                        - drop autapses
+                        - drop synapses from/to background (id 0)
+
+    batch_size :    int
+                    Number of IDs to query per batch. Too large batches might
+                    lead to truncated tables.
+    live_query :    bool
+                    Whether to query against the live data or against the latest
+                    materialized table. The latter is useful if you are working
+                    with IDs that you got from another annotation table.
     dataset :       str | CloudVolume
                     Against which flywire dataset to query::
                         - "production" (current production dataset, fly_v31)
@@ -116,54 +167,79 @@ def fetch_synapses(x, pre=True, post=True, attach=True, min_score=0,
     ids = parse_root_ids(x)
 
     # Check if any of these root IDs are outdated
-    not_latest = ids[~is_latest_root(ids, dataset=dataset)]
-    if any(not_latest):
-        print(f'Root ID(s) {", ".join(not_latest.astype(str))} are outdated '
-              'and connectivity might be inaccurrate.')
+    if live_query:
+        not_latest = ids[~is_latest_root(ids, dataset=dataset)]
+        if any(not_latest):
+            print(f'Root ID(s) {", ".join(not_latest.astype(str))} are outdated '
+                  'and connectivity might be inaccurrate.')
 
-    # Now get supervoxels for these root IDs
-    # (this is a dict)
-    roots2svxl = roots_to_supervoxels(ids, dataset=dataset, progress=progress)
-    # Turn dict into array of supervoxels
-    svoxels = np.concatenate(list(roots2svxl.values()))
+    client = get_cave_client(dataset=dataset)
 
-    # Query the synapses
-    syn = spine.synapses.get_connectivity(svoxels,
-                                          locations=True,
-                                          nt_predictions=transmitters,
-                                          segmentation='flywire_supervoxels')
+    columns = ['pre_pt_root_id', 'post_pt_root_id', 'cleft_score',
+               'pre_pt_position', 'post_pt_position']
+
+    if transmitters:
+        columns += ['gaba', 'ach', 'glut', 'oct', 'ser', 'da']
+
+    if live_query:
+        func = partial(client.materialize.live_query,
+                       table=client.materialize.synapse_table,
+                       timestamp=dt.datetime.utcnow(),
+                       split_positions=True,
+                       select_columns=columns)
+    else:
+        func = partial(client.materialize.query_table,
+                       table=client.materialize.synapse_table,
+                       split_positions=True,
+                       select_columns=columns)
+
+    syn = []
+    for i in trange(0, len(ids), batch_size,
+                    desc='Fetching synapses',
+                    disable=not progress or len(ids) <= batch_size):
+        batch = ids[i:i+batch_size]
+        if post:
+            syn.append(func(filter_in_dict=dict(post_pt_root_id=batch)))
+        if pre:
+            syn.append(func(filter_in_dict=dict(pre_pt_root_id=batch)))
+
+    # Combine results from batches
+    syn = pd.concat(syn, axis=0, ignore_index=True)
+
+    # Rename some of those columns
+    syn.rename({'post_pt_root_id': 'post',
+                'pre_pt_root_id': 'pre',
+                'post_pt_position_x': 'post_x',
+                'post_pt_position_y': 'post_y',
+                'post_pt_position_z': 'post_z',
+                'pre_pt_position_x': 'pre_x',
+                'pre_pt_position_y': 'pre_y',
+                'pre_pt_position_z': 'pre_z',
+                },
+               axis=1, inplace=True)
+
+    if transmitters:
+        syn.rename({'ach': 'acetylcholine',
+                    'glut': 'glutamate',
+                    'oct': 'octopamine',
+                    'ser': 'serotonin',
+                    'da': 'dopamine'},
+                   axis=1, inplace=True)
 
     # Next we need to run some clean-up:
-    # 1. Drop below threshold connections
+    # Drop below threshold connections
     if min_score:
-        syn = syn[syn.cleft_scores >= min_score]
-    # 2. Drop connections involving 0 (background, glia)
-    syn = syn[(syn.pre != 0) & (syn.post != 0)]
+        syn = syn[syn.cleft_score >= min_score]
+
+    if clean:
+        # Drop autapses
+        syn = syn[syn.pre != syn.post]
+        # Drop connections involving 0 (background, glia)
+        syn = syn[(syn.pre != 0) & (syn.post != 0)]
 
     # Avoid copy warning
     syn = syn.copy()
 
-    # Now map the supervoxels to root IDs
-    svoxels = np.unique(syn[['pre', 'post']].values.flatten())
-    roots = supervoxels_to_roots(svoxels, dataset=dataset)
-
-    dct = dict(zip(svoxels, roots))
-
-    # If any of the query root IDs are outdated, we have to make sure that
-    # we still map their supervoxels to their root ID and not the current
-    # root
-    dct.update({v: r for r in roots2svxl for v in roots2svxl[r]})
-
-    syn['pre'] = syn.pre.map(dct)
-    syn['post'] = syn.post.map(dct)
-
-    if not pre:
-        # Drop synapses where `post` is not in query IDs
-        syn = syn[syn.post.isin(ids)].copy()
-    if not post:
-        # Drop synapses where `pre` is not in query IDs
-        syn = syn[syn.pre.isin(ids)].copy()
-
     if attach and isinstance(x, navis.NeuronList):
         for n in x:
             presyn = postsyn = pd.DataFrame([])
@@ -197,19 +273,22 @@ def fetch_synapses(x, pre=True, post=True, attach=True, min_score=0,
                 dist, ix = tree.query(connectors[['x', 'y', 'z']].values)
                 connectors['node_id'] = n.nodes.node_id.values[ix]
 
+                # Add an ID column for navis' sake
+                connectors.insert(0, 'connector_id', np.arange(connectors.shape[0]))
+
             n.connectors = connectors
 
     return syn
 
 
-def fetch_adjacency(sources, targets=None, min_score=30, dataset='production',
-                    progress=True):
+def fetch_adjacency(sources, targets=None, min_score=30, live_query=True,
+                    batch_size=100, dataset='production', progress=True):
     """Fetch adjacency matrix.
 
     Parameters
     ----------
     sources :       int | list of int | Neuron/List
-                    Either flywire segment ID (i.e. root ID), a list thereof
+                    Either FlyWire segment ID (i.e. root ID), a list thereof
                     or a Neuron/List. For neurons, the ``.id`` is assumed to be
                     the root ID. If you have a neuron (in FlyWire space) but
                     don't know its ID, use :func:`fafbseg.flywire.neuron_to_segments`
@@ -223,6 +302,13 @@ def fetch_adjacency(sources, targets=None, min_score=30, dataset='production',
     min_score :     int
                     Minimum "cleft score". The default of 30 is what Buhmann et al.
                     used in the paper.
+    batch_size :    int
+                    Number of IDs to query per batch. Too large batches might
+                    lead to truncated tables.
+    live_query :    bool
+                    Whether to query against the live data or against the latest
+                    materialized table. The latter is useful if you are working
+                    with IDs that you got from another annotation table.
     dataset :       str | CloudVolume
                     Against which flywire dataset to query::
                         - "production" (current production dataset, fly_v31)
@@ -244,47 +330,50 @@ def fetch_adjacency(sources, targets=None, min_score=30, dataset='production',
     both = np.unique(np.append(sources, targets))
 
     # Check if any of these root IDs are outdated
-    not_latest = both[~is_latest_root(both, dataset=dataset)]
-    if any(not_latest):
-        print(f'Root ID(s) {", ".join(not_latest.astype(str))} are outdated '
-              'and connectivity might be inaccurrate.')
+    if live_query:
+        not_latest = both[~is_latest_root(both, dataset=dataset)]
+        if any(not_latest):
+            print(f'Root ID(s) {", ".join(not_latest.astype(str))} are outdated '
+                  'and connectivity might be inaccurrate.')
 
-    # Get supervoxel IDs
-    # (this is a dict)
-    roots2svxl = roots_to_supervoxels(both, dataset=dataset, progress=progress)
+    client = get_cave_client(dataset=dataset)
 
-    # Decide which ones to query
-    if len(sources) <= len(targets):
-        query = sources
+    columns = ['pre_pt_root_id', 'post_pt_root_id', 'cleft_score']
+    if live_query:
+        func = partial(client.materialize.live_query,
+                       table=client.materialize.synapse_table,
+                       timestamp=dt.datetime.utcnow(),
+                       select_columns=columns)
     else:
-        query = targets
+        func = partial(client.materialize.query_table,
+                       table=client.materialize.synapse_table,
+                       select_columns=columns)
 
-    # Map queries to supervoxels
-    query_svoxels = np.concatenate([roots2svxl[q] for q in query])
+    syn = []
+    for i in trange(0, len(sources), batch_size,
+                    desc='Fetching adjacency',
+                    disable=not progress or len(sources) <= batch_size):
+        source_batch = sources[i:i+batch_size]
+        for k in range(0, len(targets), batch_size):
+            target_batch = targets[k:k+batch_size]
 
-    # Query the synapses by supervoxels
-    syn = spine.synapses.get_connectivity(query_svoxels,
-                                          locations=False,
-                                          nt_predictions=False,
-                                          segmentation='flywire_supervoxels')
+            syn.append(func(filter_in_dict=dict(post_pt_root_id=target_batch,
+                                                pre_pt_root_id=source_batch)))
 
-    # Drop below-threshold synapses
-    syn = syn[syn.cleft_scores >= min_score]
+    # Combine results from batches
+    syn = pd.concat(syn, axis=0, ignore_index=True)
 
-    # Flip roots to supervoxels dict
-    svxl2roots = {sv: r for r in roots2svxl for sv in roots2svxl[r]}
+    # Rename some of those columns
+    syn.rename({'post_pt_root_id': 'post', 'pre_pt_root_id': 'pre'},
+               axis=1, inplace=True)
 
-    # Drop any synapse that does not involve source and targets
-    # Do not change the way this mapping is one - there is something funny
-    # going on with data types usin the usual `syn.pre.isin(svxl2roots)` that
-    # leads to values incorrectly being assumed to be True
-    syn['pre'] = syn.pre.map(lambda x: svxl2roots.get(x, 0)).astype(np.int64)
-    syn['post'] = syn.post.map(lambda x: svxl2roots.get(x, 0)).astype(np.int64)
-    syn = syn[(syn.pre != 0) & (syn.post != 0)]
-    syn = syn[syn.pre.isin(sources) & syn.post.isin(targets)]
+    # Next we need to run some clean-up:
+    # Drop below threshold connections
+    if min_score:
+        syn = syn[syn.cleft_score >= min_score]
 
     # Aggregate
-    cn = syn.groupby(['pre', 'post']).size().reset_index(drop=False)
+    cn = syn.groupby(['pre', 'post'], as_index=False).size()
     cn.columns = ['source', 'target', 'weight']
 
     # Pivot
@@ -296,13 +385,12 @@ def fetch_adjacency(sources, targets=None, min_score=30, dataset='production',
     return adj
 
 
-def fetch_connectivity(x, clean=True, style='catmaid', min_score=30,
+def fetch_connectivity(x, clean=True, style='simple', min_score=30,
                        upstream=True, downstream=True, transmitters=False,
+                       batch_size=100, live_query=True,
                        dataset='production', progress=True):
     """Fetch Buhmann et al. (2019) connectivity for given neuron(s).
 
-    Uses a service on spine.janelia.org hosted by Eric Perlman and Davi Bock.
-
     Parameters
     ----------
     x :             int | list of int | Neuron/List
@@ -326,7 +414,7 @@ def fetch_connectivity(x, clean=True, style='catmaid', min_score=30,
                     Whether to fetch upstream connectivity of ```x``.
     downstream :    bool
                     Whether to fetch downstream connectivity of ```x``.
-    transmitter :   bool
+    transmitters :  bool
                     If True, will attach the best guess for the transmitter
                     for a given connection based on the predictions in Eckstein
                     et al (2020). IMPORTANT: the predictions are based solely on
@@ -336,6 +424,13 @@ def fetch_connectivity(x, clean=True, style='catmaid', min_score=30,
                     of salt - in particular for weak connections!
                     To get the "full" predictions see
                     :func:`fafbseg.flywire.predict_transmitter`.
+    batch_size :    int
+                    Number of IDs to query per batch. Too large batches might
+                    lead to truncated tables.
+    live_query :    bool
+                    Whether to query against the live data or against the latest
+                    materialized table. The latter is useful if you are working
+                    with IDs that you got from another annotation table.
     dataset :       str | CloudVolume
                     Against which flywire dataset to query::
                         - "production" (current production dataset, fly_v31)
@@ -355,53 +450,67 @@ def fetch_connectivity(x, clean=True, style='catmaid', min_score=30,
     ids = parse_root_ids(x)
 
     # Check if any of these root IDs are outdated
-    not_latest = ids[~is_latest_root(ids, dataset=dataset)]
-    if any(not_latest):
-        print(f'Root ID(s) {", ".join(not_latest.astype(str))} are outdated '
-              'and connectivity might be inaccurrate.')
+    if live_query:
+        not_latest = ids[~is_latest_root(ids, dataset=dataset)]
+        if any(not_latest):
+            print(f'Root ID(s) {", ".join(not_latest.astype(str))} are outdated '
+                  'and live connectivity might be inaccurrate.')
 
     if transmitters and style == 'catmaid':
         raise ValueError('`style` must be "simple" when asking for transmitters')
 
-    # Now get supervoxels for these root IDs
-    # (this is a dict)
-    roots2svxl = roots_to_supervoxels(ids, dataset=dataset, progress=progress)
-    # Turn dict into array of supervoxels
-    svoxels = np.concatenate(list(roots2svxl.values()))
+    client = get_cave_client(dataset=dataset)
 
-    # Query the synapses
-    syn = spine.synapses.get_connectivity(svoxels,
-                                          segmentation='flywire_supervoxels',
-                                          nt_predictions=transmitters)
+    columns = ['pre_pt_root_id', 'post_pt_root_id', 'cleft_score']
 
-    if not upstream:
-        syn = syn[~syn.post.isin(svoxels)]
-    if not downstream:
-        syn = syn[~syn.pre.isin(svoxels)]
+    if transmitters:
+        columns += ['gaba', 'ach', 'glut', 'oct', 'ser', 'da']
+
+    if live_query:
+        func = partial(client.materialize.live_query,
+                       table=client.materialize.synapse_table,
+                       timestamp=dt.datetime.utcnow(),
+                       select_columns=columns)
+    else:
+        func = partial(client.materialize.query_table,
+                       table=client.materialize.synapse_table,
+                       select_columns=columns)
+
+    syn = []
+    for i in trange(0, len(ids), batch_size,
+                    desc='Fetching connectivity',
+                    disable=not progress or len(ids) <= batch_size):
+        batch = ids[i:i+batch_size]
+        if upstream:
+            syn.append(func(filter_in_dict=dict(post_pt_root_id=batch)))
+        if downstream:
+            syn.append(func(filter_in_dict=dict(pre_pt_root_id=batch)))
+
+    # Combine results from batches
+    syn = pd.concat(syn, axis=0, ignore_index=True)
+
+    # Rename some of those columns
+    syn.rename({'post_pt_root_id': 'post', 'pre_pt_root_id': 'pre'},
+               axis=1, inplace=True)
+
+    if transmitters:
+        syn.rename({'ach': 'acetylcholine',
+                    'glut': 'glutamate',
+                    'oct': 'octopamine',
+                    'ser': 'serotonin',
+                    'da': 'dopamine'},
+                   axis=1, inplace=True)
 
     # Next we need to run some clean-up:
-    # 1. Drop below threshold connections
+    # Drop below threshold connections
     if min_score:
-        syn = syn[syn.cleft_scores >= min_score]
-    # 2. Drop connections involving 0 (background, glia)
-    syn = syn[(syn.pre != 0) & (syn.post != 0)]
+        syn = syn[syn.cleft_score >= min_score]
 
-    # Avoid copy warning
-    syn = syn.copy()
-
-    # Now map the supervoxels to root IDs
-    svoxels = np.unique(syn[['pre', 'post']].values.flatten())
-    roots = supervoxels_to_roots(svoxels, dataset=dataset)
-
-    dct = dict(zip(svoxels, roots))
-
-    # If any of the query root IDs are outdated, we have to make sure that
-    # we still map their supervoxels to their root ID and not the current
-    # root
-    dct.update({v: r for r in roots2svxl for v in roots2svxl[r]})
-
-    syn['pre'] = syn.pre.map(dct)
-    syn['post'] = syn.post.map(dct)
+    if clean:
+        # Drop autapses
+        syn = syn[syn.pre != syn.post]
+        # Drop connections involving 0 (background, glia)
+        syn = syn[(syn.pre != 0) & (syn.post != 0)]
 
     # Turn into connectivity table
     cn_table = syn.groupby(['pre', 'post'], as_index=False).size().rename({'size': 'weight'}, axis=1)
@@ -411,8 +520,12 @@ def fetch_connectivity(x, clean=True, style='catmaid', min_score=30,
         cn_table = catmaid_table(cn_table, query_ids=ids)
     else:
         cn_table.sort_values('weight', ascending=False, inplace=True)
+        cn_table.reset_index(drop=True, inplace=True)
 
     if transmitters:
+        # Avoid copy warning
+        syn = syn.copy()
+
         # Generate per-neuron predictions
         pred = collapse_nt_predictions(syn, single_pred=True, id_col='pre')
 
