@@ -17,6 +17,7 @@ import json
 import navis
 import os
 
+from caveclient import CAVEclient
 from pathlib import Path
 from importlib import reload
 
@@ -26,13 +27,48 @@ import numpy as np
 from .. import utils
 
 
-__all__ = ['set_chunkedgraph_secret', 'get_chunkedgraph_secret']
+__all__ = ['set_chunkedgraph_secret', 'get_chunkedgraph_secret',
+           'get_cave_client']
 
 FLYWIRE_DATASETS = {'production': 'fly_v31',
                     'sandbox': 'fly_v26'}
 
+CAVE_DATASETS = {'production': 'flywire_fafb_production',
+                 'sandbox': 'flywire_fafb_sandbox'}
 
-def get_chunkedgraph_secret(domain='prodv1.flywire-daf.com'):
+# Initialize without a volume
+fw_vol = None
+cave_clients = {}
+
+
+def get_cave_client(dataset='production', token=None, force_new=False):
+    """Get CAVE client.
+
+    Parameters
+    ----------
+    dataset :   str
+                Data set to create client for.
+    token :     str, optional
+                Your chunked graph secret (i.e. "CAVE secret"). If not provided
+                will try reading via cloud-volume.
+
+    Returns
+    -------
+    CAVEclient
+
+    """
+    if not token:
+        token = get_chunkedgraph_secret()
+
+    datastack = CAVE_DATASETS.get(dataset, dataset)
+
+    if datastack not in cave_clients or force_new:
+        cave_clients[datastack] = CAVEclient(datastack, auth_token=token)
+
+    return cave_clients[datastack]
+
+
+def get_chunkedgraph_secret(domain='prod.flywire-daf.com'):
     """Get chunked graph secret.
 
     Parameters
@@ -47,8 +83,7 @@ def get_chunkedgraph_secret(domain='prodv1.flywire-daf.com'):
 
     """
     if hasattr(cv.secrets, 'cave_credentials'):
-        token = cv.secrets.cave_credentials('prodv1.flywire-daf.com'
-                                            ).get('token', None)
+        token = cv.secrets.cave_credentials(domain).get('token', None)
         if not token:
             raise ValueError(f'No chunkedgraph secret for domain {domain} '
                              'found. Please see '
@@ -65,7 +100,7 @@ def get_chunkedgraph_secret(domain='prodv1.flywire-daf.com'):
 
 
 def set_chunkedgraph_secret(token, filepath=None,
-                            domain='prodv1.flywire-daf.com'):
+                            domain='prod.flywire-daf.com'):
     """Set chunked graph secret (called "cave credentials" now).
 
     Parameters
@@ -150,7 +185,7 @@ def parse_volume(vol, **kwargs):
             # vol = f'graphene://https://prodv1.flywire-daf.com/segmentation/1.0/{vol}'
 
             # This is the new url
-            vol = f'graphene://https://prodv1.flywire-daf.com/segmentation/table/{vol}'
+            vol = f'graphene://https://prod.flywire-daf.com/segmentation/table/{vol}'
 
             # This might eventually become the new url
             # vol = f'graphene://https://prodv1.flywire-daf.com/segmentation_proc/table/{vol}'
@@ -179,7 +214,3 @@ def parse_volume(vol, **kwargs):
     else:
         fw_vol = vol
     return fw_vol
-
-
-# Initialize without a volume
-fw_vol = None
